/*
 * Copyright 2012 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.api.animation.player;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.robokind.api.animation.Animation;
import org.robokind.api.animation.library.AnimationLibrary;
import org.robokind.api.common.config.VersionProperty;
import org.robokind.api.common.triggering.Trigger;
import org.robokind.api.common.triggering.TriggerHandler;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class AnimationTriggerHandler implements TriggerHandler{
    private final static Logger theLogger = 
            Logger.getLogger(AnimationTriggerHandler.class.getName());
    
    public final static String ANIMATION_TRIGGER_TYPE = "Animation";
    public final static String PROP_ANIMATION_NAME = "animationName";
    public final static String PROP_ANIMATION_VERSION = "animationVersion";
    
    private AnimationLibrary myLibrary;
    private AnimationPlayer myPlayer;
    
    public AnimationTriggerHandler(AnimationLibrary library, AnimationPlayer player){
        myLibrary = library;
        myPlayer = player;
    }
    
    public void setAnimationLibrary(AnimationLibrary library){
        myLibrary = library;
    }
    
    public void setPlayer(AnimationPlayer player){
        myPlayer = player;
    }
    
    @Override
    public void handleTrigger(Trigger trigger) {
        if(myLibrary == null || myPlayer == null){
            theLogger.warning("Ignoring Trigger.  "
                    + "Library and Player must both be non null.");
            return;
        }
        if(trigger == null){
            throw new NullPointerException();
        }
        if(!ANIMATION_TRIGGER_TYPE.equals(trigger.getTriggerType())){
            theLogger.log(Level.WARNING, 
                    "Invalid TriggerType.  Expected: {0}, Found: {1}", 
                    new Object[]{
                        ANIMATION_TRIGGER_TYPE, trigger.getTriggerType()});
            return;
        }
        Properties props = trigger.getTriggerProperties();
        if(props == null){
            throw new NullPointerException("No animation version property");
        }
        String name = props.getProperty(PROP_ANIMATION_NAME);
        String number = props.getProperty(PROP_ANIMATION_VERSION);
        if(name == null || number == null){
            throw new NullPointerException();
        }
        VersionProperty vp = new VersionProperty(name, number);
        playAnimation(vp);
    }
    
    private void playAnimation(VersionProperty vp){
        Animation anim = myLibrary.getAnimation(vp);
        if(anim == null){
            theLogger.log(Level.WARNING, 
                    "Unable to find Animation with VersionProperty: {0}.", 
                    vp.display());
            return;
        }
        myPlayer.playAnimation(anim);
    }

    @Override
    public String getTriggerType() {
        return ANIMATION_TRIGGER_TYPE;
    }
    
}
